#include "defines.h"
#include "register.h"
#include "serial.h"

enum {
	INDEX_UART_0,
	INDEX_UART_1,
	INDEX_UART_2,
	INDEX_UART_NUM
};

#define V850ES_JG2_UAnCTL1_9600  (0x03)
#define V850ES_JG2_UAnCTL2_9600  (0x82)

/* ǥХ */
int serial_init(int index)
{
  if (index < 0 || index >= INDEX_UART_NUM) {
    return 1;
  }

  switch (index) {
  case INDEX_UART_0:
    V850ES_JG2_UA0CTL0 = BIT_CLEAR_ALL;
    V850ES_JG2_PMC3 |= (V850ES_JG2_PMC3_PMC31 | V850ES_JG2_PMC3_PMC30);
    V850ES_JG2_PFC3 = BIT_CLEAR_ALL;
    V850ES_JG2_UA0CTL0 = (V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    V850ES_JG2_UA0OPT0 = (V850ES_JG2_UAnOPT0_UAnSLS2 | V850ES_JG2_UAnOPT0_UAnSLS0);
    V850ES_JG2_UA0CTL1 = V850ES_JG2_UAnCTL1_9600;
    V850ES_JG2_UA0CTL2 = V850ES_JG2_UAnCTL2_9600;
    V850ES_JG2_UA0CTL0 = (V850ES_JG2_UAnCTL0_UAnPWR | V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    break;

  case INDEX_UART_1:
    V850ES_JG2_UA1CTL0 = BIT_CLEAR_ALL;
    V850ES_JG2_PMC9 |= (V850ES_JG2_PMC9_PMC91 | V850ES_JG2_PMC9_PMC90);
    V850ES_JG2_PFCE9 |= (V850ES_JG2_PFCE9_PFCE91 | V850ES_JG2_PFCE9_PFCE90);
    V850ES_JG2_PFC9 = BIT_CLEAR_ALL;
    V850ES_JG2_UA1CTL0 = (V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    V850ES_JG2_UA1OPT0 = (V850ES_JG2_UAnOPT0_UAnSLS2 | V850ES_JG2_UAnOPT0_UAnSLS0);
    V850ES_JG2_UA1CTL1 = V850ES_JG2_UAnCTL1_9600;
    V850ES_JG2_UA1CTL2 = V850ES_JG2_UAnCTL2_9600;
    V850ES_JG2_UA1CTL0 = (V850ES_JG2_UAnCTL0_UAnPWR | V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    break;

  case INDEX_UART_2:
    V850ES_JG2_UA2CTL0 = BIT_CLEAR_ALL;
    V850ES_JG2_PMC3 |= (V850ES_JG2_PMC3_PMC39 | V850ES_JG2_PMC3_PMC38);
    V850ES_JG2_PFC3 = BIT_CLEAR_ALL;
    V850ES_JG2_UA2CTL0 = (V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    V850ES_JG2_UA2OPT0 = (V850ES_JG2_UAnOPT0_UAnSLS2 | V850ES_JG2_UAnOPT0_UAnSLS0);
    V850ES_JG2_UA2CTL1 = V850ES_JG2_UAnCTL1_9600;
    V850ES_JG2_UA2CTL2 = V850ES_JG2_UAnCTL2_9600;
    V850ES_JG2_UA2CTL0 = (V850ES_JG2_UAnCTL0_UAnPWR | V850ES_JG2_UAnCTL0_UAnDIR | V850ES_JG2_UAnCTL0_UAnCL);
    break;
  }

  return 0;
}

/* ǽ */
int serial_is_send_enable(int index)
{
  if (index < 0 || index >= INDEX_UART_NUM) {
    return 1;
  }

  switch (index) {
  case INDEX_UART_0:
    return (!(V850ES_JG2_UA0STR & V850ES_JG2_UAnSTR_UAnTSF));
    break;

  case INDEX_UART_1:
    return (!(V850ES_JG2_UA1STR & V850ES_JG2_UAnSTR_UAnTSF));
    break;

  case INDEX_UART_2:
    return (!(V850ES_JG2_UA2STR & V850ES_JG2_UAnSTR_UAnTSF));
    break;
  }

  return 1;
}

/* ʸ */
int serial_send_byte(int index, unsigned char c)
{
  if (index < 0 || index >= INDEX_UART_NUM) {
    return 1;
  }

  /* ǽˤʤޤԤ */
  while (!serial_is_send_enable(index)) {
    ;
  }

  switch (index) {
  case INDEX_UART_0:
    V850ES_JG2_UA0CTL0 |= V850ES_JG2_UAnCTL0_UAnTXE;
    V850ES_JG2_UA0TX = c;
    V850ES_JG2_UA0STR &= ~(V850ES_JG2_UAnSTR_UAnTSF);
   break;
  case INDEX_UART_1:
    V850ES_JG2_UA1CTL0 |= V850ES_JG2_UAnCTL0_UAnTXE;
    V850ES_JG2_UA1TX = c;
    V850ES_JG2_UA1STR &= ~(V850ES_JG2_UAnSTR_UAnTSF);
    break;
  case INDEX_UART_2:
    V850ES_JG2_UA2CTL0 |= V850ES_JG2_UAnCTL0_UAnTXE;
    V850ES_JG2_UA2TX = c;
    V850ES_JG2_UA2STR &= ~(V850ES_JG2_UAnSTR_UAnTSF);
    break;
  }

  return 0;
}

/* ǽ */
int serial_is_recv_enable(int index)
{
  if (index < 0 || index >= INDEX_UART_NUM) {
    return 1;
  }

  switch (index) {
  case INDEX_UART_0:
    return ((V850ES_JG2_UA0RIC & V850ES_JG2_UA0RIC_UA0RIF));
    break;

  case INDEX_UART_1:
    return ((V850ES_JG2_UA1RIC & V850ES_JG2_UA1RIC_UA1RIF));
    break;

  case INDEX_UART_2:
    return ((V850ES_JG2_UA2RIC & V850ES_JG2_UA2RIC_UA2RIF));
    break;
  }

  return 1;
}

/* ʸ */
unsigned char serial_recv_byte(int index)
{
  unsigned char c = '\0';
  /* dummy */

  if (index < 0 || index >= INDEX_UART_NUM) {
    return '\0';
  }

  switch (index) {
  case INDEX_UART_0:
    V850ES_JG2_UA0CTL0 |= V850ES_JG2_UAnCTL0_UAnRXE;
   break;
  case INDEX_UART_1:
    V850ES_JG2_UA1CTL0 |= V850ES_JG2_UAnCTL0_UAnRXE;
    break;
  case INDEX_UART_2:
    V850ES_JG2_UA2CTL0 |= V850ES_JG2_UAnCTL0_UAnRXE;
    break;
  }

  /* ǽˤʤޤԤ */
  while (!serial_is_recv_enable(index)) {
    ;
  }

  switch (index) {
  case INDEX_UART_0:
    c = V850ES_JG2_UA0RX;
    V850ES_JG2_UA0RIC &= ~(V850ES_JG2_UA0RIC_UA0RIF);
   break;
  case INDEX_UART_1:
    c = V850ES_JG2_UA1RX;
    V850ES_JG2_UA1RIC &= ~(V850ES_JG2_UA1RIC_UA1RIF);
    break;
  case INDEX_UART_2:
    c = V850ES_JG2_UA2RX;
    V850ES_JG2_UA2RIC &= ~(V850ES_JG2_UA2RIC_UA2RIF);
    break;
  }

  return c;
}
